@@ -31,7 +31,7 @@ except ImportError:
     pep8 = None
 
 
-__version__ = '0.6.4'
+__version__ = '0.6.5'
 
 
 PEP8_BIN = 'pep8'
@@ -323,8 +323,6 @@ class FixPEP8(object):
                                                       newline=self.newline)
             return
         else:
-            # FIXME: lazy implementation
-            #RETURN_COLUMN = 75
             indent = _get_indentation(target)
             source = target[len(indent):]
             sio = StringIO(target)
@@ -344,9 +342,11 @@ class FixPEP8(object):
             # my_long_function_name(x, y,
             #     z, ...)
             candidate0 = _shorten_line(_tokens, source, target, indent,
-                                       self.indent_word, reverse=False)
+                                       self.indent_word, reverse=False,
+                                       newline=self.newline)
             candidate1 = _shorten_line(_tokens, source, target, indent,
-                                       self.indent_word, reverse=True)
+                                       self.indent_word, reverse=True,
+                                       newline=self.newline)
             if candidate0 and candidate1:
                 if candidate0.split(self.newline)[0].endswith('('):
                     self.source[line_index] = candidate0
@@ -360,19 +360,6 @@ class FixPEP8(object):
                 # Otherwise both don't work
                 return []
 
-        # FIXME: disable now
-        #for offset in range(50):
-        #    fixed = "%s" % source[:RETURN_COLUMN - len(indent) - offset] + \
-        #            " \\\n" + indent + "    " + \
-        #            source[RETURN_COLUMN - len(indent) - offset:]
-        #    try:
-        #        ret = compile_command(fixed)
-        #    except SyntaxError:
-        #        continue
-        #    if ret:
-        #        self.source[line_index] = indent + fixed
-        #        break
-
     def fix_e502(self, result):
         """Remove extraneous escape of newline."""
         line_index = result['line'] - 1
@@ -741,7 +728,7 @@ def _fix_basic_raise(line, newline):
 
 
 def _shorten_line(tokens, source, target, indentation, indent_word,
-                  reverse=False):
+                  reverse=False, newline=LF):
     """Separate line at OPERATOR."""
     if reverse:
         tokens.reverse()
@@ -766,7 +753,7 @@ def _shorten_line(tokens, source, target, indentation, indent_word,
             # Don't modify if lines are not short enough
             if len(first) > MAX_LINE_WIDTH or len(second) > MAX_LINE_WIDTH:
                 continue
-            fixed = first + "\n" + second
+            fixed = first + newline + second
             try:
                 ret = compile(fixed, '<string>', 'exec')
             except SyntaxError:
