@@ -60,6 +60,39 @@ class TestUtils(unittest.TestCase):
         source = ["print 1\r\n", "print 2\r", "print3\r\n"]
         self.assertEqual(autopep8.CRLF, autopep8._find_newline(source))
 
+    def test_shorten_comment(self):
+        self.assertEqual('# ' + 'x' * 77 + '\n',
+                         autopep8.shorten_comment('# ' + 'x' * 100 + '\n',
+                                                  '\n'))
+
+    def test_detect_encoding(self):
+        self.assertEqual(
+            'utf-8',
+            autopep8.detect_encoding(
+                os.path.join(ROOT_DIR, 'setup.py')))
+
+    def test_read_from_filename_with_bad_encoding(self):
+        """Bad encoding should not cause an exception."""
+        self.assertEqual(
+            '# -*- coding: zlatin-1 -*-\n',
+            autopep8.read_from_filename(
+                os.path.join(ROOT_DIR, 'test', 'bad_encoding.py')))
+
+    def test_read_from_filename_with_bad_encoding2(self):
+        """Bad encoding should not cause an exception."""
+        self.assertTrue(autopep8.read_from_filename(
+            os.path.join(ROOT_DIR, 'test', 'bad_encoding2.py')))
+
+    def test_fix_whitespace(self):
+        self.assertEqual(
+            'a b',
+            autopep8.fix_whitespace('a    b', offset=1, replacement=' '))
+
+    def test_fix_whitespace_with_tabs(self):
+        self.assertEqual(
+            'a b',
+            autopep8.fix_whitespace('a\t  \t  b', offset=1, replacement=' '))
+
 
 class TestFixPEP8Error(unittest.TestCase):
 
@@ -255,6 +288,12 @@ while True:
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e203_semicolon(self):
+        line = "print(a, end=' ') ; nl = 0\n"
+        fixed = "print(a, end=' '); nl = 0\n"
+        self._inner_setup(line, options='--select=E203')
+        self.assertEqual(self.result, fixed)
+
     def test_e211(self):
         line = "d = [1, 2, 3]\nprint d  [0]\n"
         fixed = "d = [1, 2, 3]\nprint d[0]\n"
@@ -384,7 +423,7 @@ class Foo(object):
     def test_e251_with_escaped_newline(self):
         line = "1\n\n\ndef a(arg=\\\n1):\n    print(arg)\n"
         fixed = "1\n\n\ndef a(arg=1):\n    print(arg)\n"
-        self._inner_setup(line)
+        self._inner_setup(line, options='--select=E251')
         self.assertEqual(self.result, fixed)
 
     def test_e261(self):
@@ -533,6 +572,101 @@ class Foo(object):
 """def d():
     print(111, 111, 111, 111, 222, 222, 222, 222, 222, 222, 222, 222,
          222, 333, 333, 333, 333)
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_more_complicated(self):
+        line = \
+"""blahblah = os.environ.get('blahblah') or os.environ.get('blahblahblah') or os.environ.get('blahblahblahblah')
+"""
+        fixed = \
+"""blahblah = os.environ.get('blahblah') or os.environ.get(
+    'blahblahblah') or os.environ.get('blahblahblahblah')
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_skip_even_more_complicated(self):
+        line = \
+"""if True:
+    if True:
+        if True:
+            blah = blah.blah_blah_blah_bla_bl(blahb.blah, blah.blah,
+                                              blah=blah.label, blah_blah=blah_blah,
+                                              blah_blah2=blah_blah)
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
+    def test_e501_prefer_to_break_at_begnning(self):
+        """We prefer not to leave part of the arguments hanging."""
+        line = \
+"""looooooooooooooong = foo(one, two, three, four, five, six, seven, eight, nine, ten)
+"""
+        fixed = \
+"""looooooooooooooong = foo(
+    one, two, three, four, five, six, seven, eight, nine, ten)
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_do_not_break_on_keyword(self):
+        # We don't want to put a newline after equals for keywords as this
+        # violates PEP 8.
+        line = \
+"""if True:
+    long_variable_name = tempfile.mkstemp(prefix='abcdefghijklmnopqrstuvwxyz0123456789')
+"""
+        fixed = \
+"""if True:
+    long_variable_name = tempfile.mkstemp(
+        prefix='abcdefghijklmnopqrstuvwxyz0123456789')
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_do_not_break_if_useless(self):
+        line = \
+"""
+123
+('bbb', 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa')
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, line)
+
+    def test_e501_with_comment(self):
+        line = """123
+                        # This is a long comment that should be wrapped. I will wrap it using textwrap.
+"""
+        fixed = """123
+                        # This is a long comment that should be wrapped. I will
+                        # wrap it using textwrap.
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_with_more_comments(self):
+        line = """123
+                        # This is a long comment that should be wrapped. I will wrap it using textwrap.
+                        # This is the next line.
+"""
+        fixed = """123
+                        # This is a long comment that should be wrapped. I will
+                        # wrap it using textwrap.
+                        # This is the next line.
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
+    def test_e501_should_cut_comment_pattern(self):
+        line = """123
+# -- Randy's datastructure -----------------------------------------------------
+321
+"""
+        fixed = """123
+# -- Randy's datastructure ----------------------------------------------------
+321
 """
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
@@ -555,6 +689,21 @@ class Foo(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_more_complicated(self):
+        line = """\
+def foo():
+    if bar : bar+=1;  bar=bar*bar   ; return bar
+"""
+        fixed = """\
+def foo():
+    if bar:
+        bar += 1
+        bar = bar * bar
+        return bar
+"""
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
     def test_e702_with_semicolon_in_string(self):
         line = 'print(";");\n'
         fixed = 'print(";")\n'
@@ -575,6 +724,18 @@ class Foo(object):
         self._inner_setup(line)
         self.assertEqual(self.result, fixed)
 
+    def test_e702_with_semicolon_after_string(self):
+        line = """
+raise IOError('abc '
+              'def.');
+""".lstrip()
+        fixed = """
+raise IOError('abc '
+              'def.')
+""".lstrip()
+        self._inner_setup(line)
+        self.assertEqual(self.result, fixed)
+
 
 class TestFixPEP8Warning(unittest.TestCase):
 
@@ -909,6 +1070,11 @@ class TestOptions(unittest.TestCase):
         self._inner_setup(line, ['--pep8-passes', '0'])
         self.assertEqual(self.result, fixed)
 
+    def test_pep8_ignore(self):
+        line = "'abc'  \n"
+        self._inner_setup(line, ['--ignore=E,W'])
+        self.assertEqual(self.result, line)
+
     @py27_and_above
     def test_help(self):
         p = Popen(list(AUTOPEP8_CMD_TUPLE) + ['-h'],
@@ -953,34 +1119,20 @@ class TestSpawnPEP8Process(unittest.TestCase):
         f = open(self.tempfile[1], 'w')
         f.write(line)
         f.close()
-        cmd = list(AUTOPEP8_CMD_TUPLE)
-        cmd.extend(options.split())
-        cmd.append(self.tempfile[1])
+        opts, _ = autopep8.parse_args(options.split() + [self.tempfile[1]])
+        sio = StringIO()
 
-        # Put fake pep8 path in front. It will cause autopep8 to launch a
-        # a subprocess by pretending to be an older version of pep8.
-        tmp_env = os.environ.copy()
+        # Monkey patch pep8 to trigger spawning
+        original_pep8 = autopep8.pep8
+        try:
+            autopep8.pep8 = None
+            autopep8.fix_file(filename=self.tempfile[1],
+                              opts=opts,
+                              output=sio)
+        finally:
+            autopep8.pep8 = original_pep8
 
-        if 'PYTHONPATH' in os.environ:
-            old_python_path = os.environ['PYTHONPATH']
-        else:
-            old_python_path = ''
-
-        tmp_env['PYTHONPATH'] = (os.path.join(ROOT_DIR, 'test',
-                                              'fake_pep8', 'site-packages') +
-                                 ':' + old_python_path)
-
-        if 'PATH' in os.environ:
-            old_path = os.environ['PATH']
-        else:
-            old_path = ''
-
-        tmp_env['PATH'] = (os.path.join(ROOT_DIR, 'test',
-                                        'fake_pep8', 'bin') +
-                           ':' + old_path)
-
-        p = Popen(cmd, stdout=PIPE, env=tmp_env)
-        self.result = p.communicate()[0].decode('utf8')
+        self.result = sio.getvalue()
 
     def test_basic(self):
         line = "print('abc' )    \n1*1\n"
